package com.zy.usercenterservice.controller;

import com.google.gson.Gson;
import com.zy.commonutils.exception.GuliException;
import com.zy.servicebase.utils.JwtUtils;
import com.zy.usercenterservice.entity.UcenterMember;
import com.zy.usercenterservice.service.UcenterMemberService;
import com.zy.usercenterservice.utils.ConstanWxtiesUtil;
import com.zy.usercenterservice.utils.HttpClientUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import javax.servlet.http.HttpSession;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;

@Controller
@RequestMapping("/api/ucenter/wx")
public class WxApiController {
    @Autowired
    private UcenterMemberService ucenterMemberService;

    // 获取 扫描人 信息，添加数据
    @GetMapping("callback")
    public String callback(String code, String state, HttpSession session){
        try {
        //1、得到授权临时票据code

        //2、拿着code请求 微信的固定地址，得到两个值access_token和openid
            String baseAccessTokenUrl = "https://api.weixin.qq.com/sns/oauth2/access_token" +
                    "?appid=%s" +
                    "&secret=%s" +
                    "&code=%s" +
                    "&grant_type=authorization_code";
            String accessTokenUrl = String.format(baseAccessTokenUrl,
                    ConstanWxtiesUtil.WX_OPEN_APP_ID,
                    ConstanWxtiesUtil.WX_OPEN_APP_SECRET,
                    code);
            //请求这个拼接好的地址，得到返回两个值access_token和openid
            //使用httpclient发送请求，得到返回结果
            String accessTokenInfo = HttpClientUtils.get(accessTokenUrl);

            //使用gson转换工具Gson
            Gson gson = new Gson();
            HashMap mapAccessToken = gson.fromJson(accessTokenInfo, HashMap.class);
            String access_token = (String)mapAccessToken.get("access_token");
            String openid = (String)mapAccessToken.get("openid");

            //判断该微信信息是否注册过
            UcenterMember menber = ucenterMemberService.getMenberByOperid(openid);
            if (menber == null){
                // 拿着access_token和openid，再去请求微信提供的固定地址，获取扫描人信息
                // 访问微信的资源服务器，获取用户信息
                String baseUserInfoUrl = "https://api.weixin.qq.com/sns/userinfo" +
                        "?access_token=%s" +
                        "&openid=%s";

                // 再次拼接微信地址
                String userInfoUrl = String.format(baseUserInfoUrl, access_token, openid);

                // 发送请求 获取 个人信息
                String userInfo = HttpClientUtils.get(userInfoUrl);

                // 获取的微信个人信息json信息进行转换
                HashMap userInfoMap = gson.fromJson(userInfo, HashMap.class);
                String nickname = (String)userInfoMap.get("nickname");
                String headImgUrl = (String)userInfoMap.get("headimgurl");
                //把微信信息注册到数据库中
                menber = new UcenterMember();
                menber.setNickname(nickname);
                menber.setOpenid(openid);
                menber.setAvatar(headImgUrl);
                ucenterMemberService.save(menber);
            }
            //使用jwt生成token字符串
            String jwtToken = JwtUtils.getJwtToken(menber.getId(), menber.getNickname());
            //返回首页面
            return "redirect:http://localhost:3000?token=" + jwtToken;
        } catch (Exception e) {
            e.printStackTrace();
            throw new GuliException(20001,"登录失败");
        }
    }

    // 生成微信二维码
    @GetMapping("login")
    public String getWxCode(){
        // 微信开放平台授权baseUrl
        //? %s相当于占位符，可以填充参数
        String baseUrl = "https://open.weixin.qq.com/connect/qrconnect" +
                "?appid=%s" +
                "&redirect_uri=%s" +
                "&response_type=code" +
                "&scope=snsapi_login" +
                "&state=%s" +
                "#wechat_redirect";
        // 授权码需要传入加密过的URL,必须使用
        String redirectUrl = ConstanWxtiesUtil.WX_OPEN_REDIRECT_URL;
        //获取业务服务器重定向地址
        try {
            redirectUrl = URLEncoder.encode(redirectUrl,"utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        // 设置 %s 参数值
        String url = String.format(baseUrl, ConstanWxtiesUtil.WX_OPEN_APP_ID, redirectUrl, "zy");
        //请求微信地址,重定向的方式
        return "redirect:" + url;
    }
}
